#!/bin/bash

export LC_ALL=ja_JP.UTF-8

usage_exit() {
    printf "%s [-Ajaszweid] [input_file]\n" $0
    echo "  少なくとも、-d以外のオプション1つは選択しなければならない。"
    echo "    S : スペースとタブを_に変換"
    echo "    A : 以下すべてのオプションをon"
    echo "    j : 半角カナを全角カナに変換。特殊文字を一般系に変換。"
    echo "    a : 英数全角を半角に変換"
    echo "    s : 全角記号を半角に変換"
    echo "    z : 全角スペースを半角スペースに変換"
    echo "    w : 連続する空白を1つにまとめる"
    echo "    e : 連続する空行を1つにまとめる"
    echo "    i : 半角英字を全て小文字に変換"
    echo "    d : デバッグオプション"
    exit
}

ALNUM_ZEN2HAN='y/ＡＢＣＤＥＦＧＨＩＪＫＬＭＮＯＰＱＲＳＴＵＶＷＸＹＺ０１２３４５６７８９/ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789/;'
SYMBOL_ZEN2HAN=$'y/｀〜！＠＃＄％＾＆＊（）−＿＝＋［｛］｝\｜；：’”，＜．＞／？￥/`~!@#$%^&*()-_=+[{]}\\|;:\'",<.>\\/?¥/;'
SPACEPACK='s/[ \t]\+/ /g;'
#SPACEPACK='s/[[:space:]]\+/ /g;'
IGNORECASE='s/\(.*\)/\L\1/g;'
ZENSPC="y/　/ /;"

SPECIAL='s/㈱/(株)/g;s/㈲/(有)/g;'
KANA_HAN2ZEN='
s/ｳﾞ/ヴ/g;
s/ｶﾞ/ガ/g; s/ｷﾞ/ギ/g; s/ｸﾞ/グ/g; s/ｹﾞ/ゲ/g; s/ｺﾞ/ゴ/g;
s/ｻﾞ/ザ/g; s/ｼﾞ/ジ/g; s/ｽﾞ/ズ/g; s/ｾﾞ/ゼ/g; s/ｿﾞ/ゾ/g;
s/ﾀﾞ/ダ/g; s/ﾁﾞ/ヂ/g; s/ﾂﾞ/ヅ/g; s/ﾃﾞ/デ/g; s/ﾄﾞ/ド/g;
s/ﾊﾞ/バ/g; s/ﾋﾞ/ビ/g; s/ﾌﾞ/ブ/g; s/ﾍﾞ/ベ/g; s/ﾎﾞ/ボ/g;
s/ﾊﾟ/パ/g; s/ﾋﾟ/ピ/g; s/ﾌﾟ/プ/g; s/ﾍﾟ/ペ/g; s/ﾎﾟ/ポ/g;
y/ｱｲｳｴｵｶｷｸｹｺｻｼｽｾｿﾀﾁﾂﾃﾄﾅﾆﾇﾈﾉﾊﾋﾌﾍﾎﾏﾐﾑﾒﾓﾔﾕﾖﾜｦﾝｧｨｩｪｫｯｬｭｮﾞﾟｰ･｡｢｣､/アイウエオカキクケコサシスセソタチツテトナニヌネノハヒフヘホマミムメモヤユヨワヲンァィゥェォッャュョ゛゜ー・。「」、/;
'
DAKUHANDAKU='
s/う゛/ゔ/g;
s/か゛/が/g; s/き゛/ぎ/g; s/く゛/ぐ/g; s/け゛/げ/g; s/こ゛/ご/g;
s/さ゛/ざ/g; s/し゛/じ/g; s/す゛/ず/g; s/せ゛/ぜ/g; s/そ゛/ぞ/g;
s/た゛/だ/g; s/ち゛/ぢ/g; s/つ゛/づ/g; s/て゛/で/g; s/と゛/ど/g;
s/は゛/ば/g; s/ひ゛/び/g; s/ふ゛/ぶ/g; s/へ゛/べ/g; s/ほ゛/ぼ/g;
s/は゜/ぱ/g; s/ひ゜/ぴ/g; s/ふ゜/ぷ/g; s/へ゜/ぺ/g; s/ほ゜/ぽ/g;
s/ウ゛/ヴ/g;
s/カ゛/ガ/g; s/キ゛/ギ/g; s/ク゛/グ/g; s/ケ゛/ゲ/g; s/コ゛/ゴ/g;
s/サ゛/ザ/g; s/シ゛/ジ/g; s/ス゛/ズ/g; s/セ゛/ゼ/g; s/ソ゛/ゾ/g;
s/タ゛/ダ/g; s/チ゛/ヂ/g; s/ツ゛/ヅ/g; s/テ゛/デ/g; s/ト゛/ド/g;
s/ハ゛/バ/g; s/ヒ゛/ビ/g; s/フ゛/ブ/g; s/ヘ゛/ベ/g; s/ホ゛/ボ/g;
s/ハ゜/パ/g; s/ヒ゜/ピ/g; s/フ゜/プ/g; s/ヘ゜/ペ/g; s/ホ゜/ポ/g;
'

EMPTYLINE2='/^$/N;/^\n$/D;'

SPC2_='s/[ \t]/_/g'

## ##
debug=0
while getopts :SAjaszweid OPT
do
  case $OPT in
    S) spc2_=${SPC2_} ;;
    A) emptyline2=${EMPTYLINE2}
       special=${SPECIAL}
       kana_han2zen=${KANA_HAN2ZEN}
       dakuhandaku=${DAKUHANDAKU}
       alnum_zen2han=${ALNUM_ZEN2HAN}
       symbol_zen2han=${SYMBOL_ZEN2HAN}
       zenspc=${ZENSPC}
       spacepack=${SPACEPACK}
       ignorecase=${IGNORECASE}
       ;;
    j) special=${SPECIAL}
       kana_han2zen=${KANA_HAN2ZEN}
       dakuhandaku=${DAKUHANDAKU}
       ;;
    a) alnum_zen2han=${ALNUM_ZEN2HAN} ;;
    s) symbol_zen2han=${SYMBOL_ZEN2HAN} ;;
    z) zenspc=${ZENSPC} ;;
    w) spacepack=${SPACEPACK} ;;
    i) ignorecase=${IGNORECASE} ;;
    e) emptyline2=${EMPTYLINE2} ;;
    d) debug=1 ;;
    *) usage_exit ;;
  esac
done
shift $((OPTIND - 1))

SED_COMMAND="${emptyline2} ${special} ${kana_han2zen} ${dakuhandaku} ${alnum_zen2han} ${symbol_zen2han} ${zenspc} ${spacepack} ${ignorecase} ${spc2_}"

spcs=$'^[ \t]*$'
[[ "${SED_COMMAND}" =~ $spcs ]] && usage_exit

[ ${debug} -gt 0 ] && echo "SED_COMMAND=${SED_COMMAND}\nAWK_COMMAND=${AWK_COMMAND}" >&2

sed -e "${SED_COMMAND}" "$@"

