#!/bin/bash
#
# Description: ロケールによって、tr文字クラスのCollationの動作を確認するスクリプト
#              言語毎のロケールを変えながら、文字クラス[=equiv=]の挙動を確認する
# Ref: https://en.wikipedia.org/wiki/List_of_Unicode_characters#Latin-1_Supplement

workaround_sed() { # すべてsedで置き換える
    sed -e "y/âàäá/aaaa/;" \
        -e "y/êéèë/eeee/;" \
        -e "y/ìïîí/iiii/;" \
        -e "y/òôöó/oooo/;" \
        -e "y/ùüûú/uuuu/;" \
        -e "y/ÿ/y/;" \
        -e "y/ç/c/;" \
        -e "y/ñ/n/;" \
        -e "s/æ/ae/g;" \
        -e "s/œ/oe/g;" \
        -e "s/ß/ss/g;" \
        -e "y/ÂÀÄÁ/AAAA/;" \
        -e "y/ÊÈËÉ/EEEE/;" \
        -e "y/ÌÏÎÍ/IIII/;" \
        -e "y/ÒÔÖÓ/OOOO/;" \
        -e "y/ÙÜÛÚ/UUUU/;" \
        -e "y/Ÿ/Y/;" \
        -e "y/Ç/C/;" \
        -e "y/Ñ/N/;" \
        -e "s/Æ/AE/g;" \
        -e "s/Œ/OE/g;" \
        -e "s/ẞ/SS/g;"
}

workaround_tr() { # https://en.wikipedia.org/wiki/List_of_Unicode_characters#Latin-1_Supplement
               #UTF-8であることを過程して、8進表記で置き換える
    tr '\340\341\342\343\344\345' "a" |
    tr '\350\351\352\353' 'e' |
    tr '\354\355\356\357' "i" |
    tr '\362\363\364\365\366' "o" |
    tr '\371\372\373\374' "u" |
    tr '\375\377' "y" |
    tr '\361' "n" |
    tr '\347' "c" |
    tr '\300\301\302\303\304\305' "A" |
    tr '\310\311\312\313' "E" |
    tr '\314\315\316\317' "I" |
    tr '\322\323\324\325\326' "O" |
    tr '\331\332\333\334' "U" |
    tr '\335' "Y" |
    tr '\321' "N" |
    tr '\307' "C"
}

current_implement() {
    tr "[=a=]" "a" |
    tr "[=e=]" "e" |
    tr "[=i=]" "i" |
    tr "[=o=]" "o" |
    tr "[=u=]" "u" |
    tr "[=y=]" "y" |
    tr "[=n=]" "n" |
    tr "[=c=]" "c" |
    tr "[=A=]" "A" |
    tr "[=E=]" "E" |
    tr "[=I=]" "I" |
    tr "[=O=]" "O" |
    tr "[=U=]" "U" |
    tr "[=Y=]" "Y" |
    tr "[=Y=]" "Y" |
    tr "[=N=]" "N" |
    tr "[=C=]" "C"
}

str=$'
French: â à ê é è ë ì ï î ò ô ù ü û ÿ æ œ ç
German: ä ö ü ß
Espagnol: á é í ó ú ñ
'
STR=$'
FRENCH: Â À Ê È Ë Ì Ï Î Ò Ô Ù Ü Û Ÿ Æ Œ Ç
GERMAN: Ä Ö Ü ẞ
ESPAGNOL: Á É Í Ó Ú Ñ
'

unset LANG LC_ALL

for l in "C" "C.UTF-8" "es_ES.UTF-8" "fr_FR.UTF-8" "de_DE.UTF-8" ; do
    echo "-------------------------------------------------"
    ### どのロケール環境変数が影響するか、コメントを着脱して確認 ##
    #export LANG="$l"        # する
    #export LC_ALL="$l"      # もちろん影響する
    export LC_CTYPE="$l"    # 表示可能文字として影響
    export LC_COLLATE="$l"  # [=equiv=] Collation に影響
    #locale                  # 必要ならロケールが反映されているかここで確認
    ##############################################################
    read -p "${l} pause:" n
    current_implement <<<"$str$STR"
    #echo "+++ Workaround ++++++++++++++++++++++++++++++++++"
    #workaround_sed <<<"$str$STR"
    #workaround_tr <<<"$str$STR"
done

#######
# Results: [=equiv=] についての考察
# macOSでは、動作する ... Ÿ は変換していないが Ÿ は、そもそもLatin-1にない
# LC_CTYPEとLC_COLLATEの2つの環境変数が反映される
#
# Ubuntuでは、欧州言語のlocaleをインストールしても機能しない
# 
# Solutions:
# どうしてもtrで解決する方法(workaround_tr)は、コード(UTF-8)を8進表記
# sedで置き換える方法(workaround_sed)
